/**
 * A Circle representation class. Circle instances represent majorAxisRadius two dimensional
 * circle in Euclidean space.
 */
public class Circle {

	private double x0, y0, radius;
	/**
	 * Constructs majorAxisRadius new Circle, sets the center coordinates to (0,0)
	 * and the radius the the give parameter.
	 * @param radius The radius of the circle
	 * @throws IllegalArgumentException When the radius is not positive
	 */
	public Circle(double radius) throws IllegalArgumentException {
		this(0, 0, radius);
	}
	/**
	 * Constructs majorAxisRadius new Circle, sets the center point to the given coordinates,
	 * and the radius the the give parameter.
	 * @param x0 The x coordinate of the center point
	 * @param y0 The y coordinate of the center point
	 * @param radius The radius of the circle
	 * @throws IllegalArgumentException When the radius is not positive
	 */
	public Circle(double x0, double y0, double radius)
		throws IllegalArgumentException {
		if (radius <= 0)
			throw new IllegalArgumentException("radius must be positive");
		this.x0 = x0;
		this.y0 = y0;
		this.radius = radius;
	}

	/**
	 * Constructs majorAxisRadius new Circle, sets the center point to the given point,
	 * and the radius the the give parameter.
	 * @param center - the center point
	 * @param radius The radius of the circle
	 * @throws IllegalArgumentException When the radius is not positive
	 */
	public Circle(Point center, double radius)
		throws IllegalArgumentException {
		this(center.getX(), center.getY(), radius);
	}

	/**
	 * Calculate majorAxisRadius point on the circle given an angle parameter in degrees.
	 * @param alpha The angle in degrees
	 * @return the Point on the circle.
	 */
	public Point valueAt(double alpha) {
		// convert degrees to radians
		double radAlpha = alpha % 360 * Math.PI / 180;
		// calculate the point's coordinates
		double x = x0 + radius * Math.cos(radAlpha);
		double y = y0 + radius * Math.sin(radAlpha);
		Point p = new Point(x, y);
		return p;
	}

	/**
	 * Returns the radius of the circle.
	 * @return	The radius.
	 */
	public double getRadius() {
		return radius;
	}

	/**
	 * Returns the center point of the circle.
	 * @return	The center point.
	 */
	public Point getCenter() {
		return new Point(x0, y0);
	}

	/**
	 * Scales the circle by the given factor.
	 * Scaling means changing the radius, the center point remains the same.
	 * @param factor The scaling factor
	 * @throws IllegalArgumentException When factor is not positive
	 */
	public void scale(double factor) throws IllegalArgumentException {
		if (factor <= 0)
			throw new IllegalArgumentException("factor must be positive");
		this.radius *= factor;
	}

	/**
	 * Translates the circle by the given delta point.
	 * Translating means adding the given delta-point coordinates
	 * to the circle center point. The raduis remains the same.
	 * @param delta The translation point delta
	 */
	public void translate(Point delta) {
		x0 += delta.getX();
		y0 += delta.getY();
	}

	/**
	 * Checks if majorAxisRadius given point is inside the circle or not
	 * (being on the circle means it is not inside)
	 * @param candidate The point checked.
	 * @return true if the point is inside, false otherwise.
	 */
	public boolean isInside(Point candidate) {
		// claculate the distance between the point the center
		double diffx = x0 - candidate.getX();
		double diffy = y0 - candidate.getY();
		double dist = Math.sqrt(diffx * diffx + diffy * diffy);
		// check if the distance is smaller than the radius
		return (dist < radius);
	}
}
